`timescale 1ns/1ns


module ControlTransfer
(
	input wire 	clk 
	,input wire rst_n

	// requests
	,input wire			request_active 	
	,input wire	[7:0]	bmRequestType	
	,input wire	[7:0]	bRequest		
	,input wire	[15:0]	wValue			
	,input wire	[15:0]	wIndex			
	,input wire	[15:0]	wLength	

	// ep0 IN  
	,output	reg	[31:0]	ep0_in_buf_data
	,output	reg			ep0_in_buf_wren
	,input	wire		ep0_in_buf_ready
	,output	reg			ep0_in_buf_data_commit
	,output	reg	[10:0]	ep0_in_buf_data_commit_len		

	// ep0 OUT	
	,input wire	[31:0]	ep0_out_buf_data		 	
	,input wire	[10:0]	ep0_out_buf_len		
	,input wire			ep0_out_buf_hasdata	
	,output reg			ep0_out_buf_rden
	,output reg			ep0_out_buf_data_ack			


);

`include "usb_descrip.vh"

localparam STANDARD = 2'b00 ;

localparam [7:0]	STANDARD_GET_STATUS			= 8'h0;
localparam [7:0]	STANDARD_CLEAR_FEAT			= 8'h1;
localparam [7:0]	STANDARD_SET_FEAT			= 8'h3;
localparam [7:0]	STANDARD_GET_DESCR			= 8'h6;
localparam [7:0]	STANDARD_SET_DESCR			= 8'h7;
localparam [7:0]	STANDARD_GET_CONFIG			= 8'h8;
localparam [7:0]	STANDARD_SET_CONFIG			= 8'h9;
localparam [7:0]	STANDARD_SET_INTERFACE		= 8'hB;
localparam [7:0]	STANDARD_SYNCH_FRAME		= 8'h12;
localparam [7:0]	STANDARD_SET_SEL			= 8'd48;
localparam [7:0]	STANDARD_GET_INTERFACE		= 8'd10;
localparam [7:0]	STANDARD_GET_CONFIGURATION 	= 8'd8;
localparam [7:0]	STANDARD_SET_IOS_DELAY		= 8'd49;
localparam [7:0]	STANDARD_SYNC_FRAME			= 8'd12;

reg wr_desc_pkt_act_d;
reg wr_ep0_act_d;
reg wr_desc_act_d;
reg wr_resp_pkt_act_d;
reg rd_ep0_pkt_act_d;
reg wr_desc_pkt_act;
reg wr_ep0_act;
reg wr_desc_act;
reg wr_resp_pkt_act;
reg rd_ep0_pkt_act;
reg wr_desc_req;
reg get_interface;
reg [DESCR_USB3_ROM_ADD_W-1:0] wr_desc_len ;
reg [DESCR_USB3_ROM_ADD_W-1:0] descr_offset ;
reg get_status ;
reg set_config ;
reg [15:0] set_config_value ;
reg get_config ;
reg set_feature ;
reg set_interface ;
reg [15:0] set_interface_num ;
reg [15:0] set_interface_alt_setting ;
reg set_sel ;
reg set_iso_delay ; 
reg [15:0] set_iso_delay_value ;
reg sync_frame ;
reg [15:0] sync_frame_endpt ;
reg [DESCR_USB3_ROM_ADD_W-1:0] wr_desc_len_latch ;
reg wr_resp_act; 
reg [15:0] wr_resp_len_latch ;
reg [10:0] wr_ep0_len_cnt ;
reg [10:0] wr_ep0_len ;
reg [DESCR_USB3_ROM_ADD_W-1:0] descr_rd_addr ;
wire [31:0] resp_data_q;
reg 	set_sel_act;
reg 	sync_frame_act;
wire ep0_out_buf_data_val = ep0_out_buf_rden ;
reg [9:0]ep0_out_buf_data_inx;
reg [7:0]u1_sel;
reg [7:0]u1_pel;
reg [15:0] u2_sel;
reg [15:0] u2_pel;
reg [15:0] frame_num;
reg rd_ep0_act;
reg [10:0] rd_ep0_len_latch;
reg [10:0] rd_ep0_len_cnt;
reg 	 rd_ep0_act_d;
reg get_descr ;


always @ ( posedge clk or negedge rst_n ) begin
	if ( !rst_n ) begin
		wr_desc_pkt_act_d <= 0 ;
		wr_ep0_act_d <= 0 ;
		wr_desc_act_d <= 0 ;
		wr_resp_pkt_act_d <= 0 ;
		rd_ep0_pkt_act_d <= 0 ;
		rd_ep0_act_d <= 0 ;
	end
	else begin
		wr_desc_pkt_act_d <= wr_desc_pkt_act ;
		wr_ep0_act_d <= wr_ep0_act ;
		wr_desc_act_d <= wr_desc_act ;
		wr_resp_pkt_act_d <= wr_resp_pkt_act ;
		rd_ep0_pkt_act_d <= rd_ep0_pkt_act ;
		rd_ep0_act_d <= rd_ep0_act ;
	end
	
end

// managing set-up pkts
always @ ( posedge clk  or negedge rst_n ) begin
	if ( !rst_n ) begin
		get_descr <= 0 ;
		get_interface <= 0 ;
		get_status <= 0 ;
		set_config <= 0 ;
		get_config <= 0 ;
		set_feature <= 0 ;
		set_interface <= 0 ;
		set_sel <= 0 ;
		set_iso_delay <= 0 ;		
	end
	else if ( request_active ) begin
		case ( bmRequestType[6:5] ) 
		STANDARD:begin
			case ( bRequest )
			STANDARD_GET_DESCR:begin
				get_descr <= 1 ;
			end
			STANDARD_CLEAR_FEAT:begin
				show_time();
				$display("host clear feature \n");
			end
			STANDARD_GET_INTERFACE:begin
				get_interface <= 1 ;
				show_time();
				$display("host get interface \n");
			end
			STANDARD_GET_STATUS:begin
				get_status <= 1 ;
				show_time();
				$display("host get status \n");
			end
			STANDARD_SET_CONFIG:begin
				set_config <= 1 ;
				set_config_value <= wValue ;
				show_time();
				$display("host set configuration \n");	
				$display("configuration value = %d \n" , wValue );					
			end
			STANDARD_GET_CONFIG:begin
				get_config <= 1 ;
				show_time(); 
				$display("host get configuration \n");					
			end
			STANDARD_SET_FEAT:begin
				set_feature <= 1 ;
				show_time();
				$display("host set feature \n");
			end
			STANDARD_SET_INTERFACE:begin
				set_interface <= 1 ;
				set_interface_num <= wIndex ;
				set_interface_alt_setting <= wValue ;
				show_time();
				$display("host set interface \n");
				$display("interface number = %d \n" , wIndex );
				$display("interface setting = %d \n" , wValue );
			end
			STANDARD_SET_SEL:begin
				set_sel <= 1 ;
				show_time();
				$display("host set sel \n");
			end
			STANDARD_SET_IOS_DELAY:begin
				set_iso_delay <= 1 ;
				set_iso_delay_value <= wValue ;
				show_time();
				$display("host set iso delay \n");
			end
			STANDARD_SYNC_FRAME:begin
				sync_frame <= 1 ;
				sync_frame_endpt <= wIndex ;
				show_time();
				$display("host sync frame , endpt number =%d" , wIndex ) ;
			end
			default:begin end
			endcase 
		end
		default:begin end
		endcase 
	end
	else begin
		get_descr <= 0 ;
		get_interface <= 0 ;
		get_status <= 0 ;
		set_config <= 0 ;
		get_config <= 0 ;
		set_feature <= 0 ;
		set_interface <= 0 ;
		set_sel <= 0 ;
		set_iso_delay <= 0 ;
	end
end


always @ ( posedge clk or negedge rst_n ) begin
	if ( !rst_n ) begin
		wr_desc_req <= 0 ;		
	end
	else if ( get_descr ) begin
		case ( wValue[15:8] ) 
		1:begin
		//DEVICE
			wr_desc_req	<= 	1 ;
			wr_desc_len <=	DESCR_USB3_DEVICE_LEN ;
			descr_offset <= DESCR_USB3_DEVICE ;
			show_time();
			$display("host requests for Device descriptor \n");
		end
		2:begin
		//CONFIGURATION
			wr_desc_req	<= 	1 ;
			wr_desc_len <=	DESCR_USB3_CONFIG_LEN ;				
			descr_offset <= DESCR_USB3_CONFIG ;
			show_time();
			$display("host requests for Config descriptor \n"); 
		end
		3:begin
		//STRING
			case ( wValue [7:0] )
			0:begin
			//STRING 0 
				wr_desc_req	<= 	1 ;
				wr_desc_len <=	DESCR_USB3_STR0_LEN ;						
				descr_offset <= DESCR_USB3_STR0 ;
			end
			238:begin
			//STRING 238
				wr_desc_req	<= 	1 ;
				wr_desc_len <=	DESCR_USB3_STR_MSFT_LEN ;						
				descr_offset <= DESCR_USB3_STR_MSFT ;
			end
			default:begin 
				wr_desc_req <= 0 ;
			end
			endcase 
		end
		//4:begin
		////INTERFACE
		//end
		//5:begin
		////ENDPOINT
		//end
		//8:begin
		////INTERFACE_POWER1
		//end
		//9:begin
		////OTG
		//end
		//10:begin
		////DEBUG
		//end
		//11:begin
		////INTERFACE_ASSOCIATION
		//end
		15:begin
		//BOS
			wr_desc_req	<= 	1 ;
			wr_desc_len <=	DESCR_USB3_BOS_LEN ;					
			descr_offset <= DESCR_USB3_BOS ;
			show_time();
			$display("host requests for Bos descriptor \n");
		end
		//16:begin
		////DEVICE CAPABILITY
		//end
		//48:begin
		////SUPERSPEED_USB_ENDPOINT_COMPANION
		//end
		//49:begin
		////SUPERSPEEDPLUS_ISOCHRONOUS_ENDPOINT_COMPANION
		//end
		default:begin 
			wr_desc_req <= 0 ;
		end
		endcase 
	end
	else begin
		wr_desc_req <= 0 ;
	end
end


// managing response data pkts

// write descriptors
always @ ( posedge clk or negedge rst_n ) begin
	if ( !rst_n ) begin
		wr_desc_act <= 0 ;
		wr_desc_pkt_act <= 0 ;
	end
	else if ( wr_desc_req ) begin
		wr_desc_act <= 1 ;		
		wr_desc_len_latch <= wr_desc_len ;
	end
	else if ( wr_desc_act ) begin 
		if ( wr_desc_pkt_act && ep0_in_buf_data_commit ) begin
			wr_desc_len_latch <= wr_desc_len_latch - ep0_in_buf_data_commit_len ;
			wr_desc_pkt_act <= 0 ;
		end
		else if ( wr_desc_len_latch > 0 ) begin
			wr_desc_pkt_act <= 1 ;
		end
		else begin
			wr_desc_act <= 0 ; 
		end
	end
end
		
// responsing requests
always @ ( posedge clk or negedge rst_n ) begin
	if ( !rst_n ) begin
		wr_resp_act <= 0 ;
		wr_resp_pkt_act <= 0 ;
	end
	else if ( get_interface ) begin
		wr_resp_act <= 1 ;
		wr_resp_len_latch <= wLength ;
	end
	else if ( get_status ) begin
		wr_resp_act <= 1 ;
		wr_resp_len_latch <= wLength ;	
	end
	else if ( get_config ) begin
		wr_resp_act <= 1 ;
		wr_resp_len_latch <= 1 ;
	end
	else if ( wr_resp_act ) begin
		if ( wr_resp_pkt_act && ep0_in_buf_data_commit ) begin
			wr_resp_len_latch <= wr_resp_len_latch - ep0_in_buf_data_commit_len ;
			wr_resp_pkt_act <= 0 ;
		end
		else if ( wr_resp_len_latch > 0 ) begin
			wr_resp_pkt_act <= 1 ;
		end
		else begin
			wr_resp_act <= 0 ;
		end
	end
end	



always @ ( posedge clk or negedge rst_n ) begin
	if ( !rst_n ) begin		
		wr_ep0_len_cnt <= 0 ;
		wr_ep0_act <= 0 ;
	end
	else if ( !wr_desc_pkt_act_d && wr_desc_pkt_act ) begin
		wr_ep0_act <= 1 ;
		wr_ep0_len <= ( wr_desc_len_latch >= 1024 ) ?  1024 : wr_desc_len_latch ;
	end
	else if ( !wr_resp_pkt_act_d && wr_resp_pkt_act ) begin
		wr_ep0_act <= 1 ; 
		wr_ep0_len <= ( wr_resp_len_latch >= 1024 ) ? 1024 : wr_resp_len_latch ;
	end
	else if ( wr_ep0_act && ep0_in_buf_ready ) begin
		wr_ep0_len_cnt <= wr_ep0_len_cnt + 4 ;
		if ( wr_ep0_len_cnt + 4 >= wr_ep0_len  ) begin
			wr_ep0_act <= 0 ;
			wr_ep0_len_cnt <= 0 ;
		end
	end
end	



always @ ( posedge clk or negedge rst_n ) begin
	if (!rst_n) begin
		ep0_in_buf_wren <= 0 ;
	end
	else if ( wr_ep0_act && ep0_in_buf_ready ) begin
		ep0_in_buf_wren <= 1 ;
	end
	else if ( wr_ep0_act_d && !wr_ep0_act ) begin
		ep0_in_buf_wren 		<= 0 ; 
		ep0_in_buf_data_commit		<= 1 ;
		ep0_in_buf_data_commit_len	<= wr_ep0_len ;
	end
	else begin
		ep0_in_buf_wren <= 0 ;
		ep0_in_buf_data_commit <= 0 ;
	end
end


always @ ( posedge clk or negedge rst_n ) begin
	if (!rst_n) begin
		descr_rd_addr <= 0 ;
	end
	else if ( wr_desc_act_d && !wr_desc_act ) begin
		descr_rd_addr <= 0 ;
	end
	else if ( wr_desc_act && wr_ep0_act ) begin
		descr_rd_addr <= descr_rd_addr + 1 ;
	end
end


reg [31:0] mem[0:127];
reg [31:0 ]descr_data_q ;
always @( posedge clk ) begin
	descr_data_q <= mem[descr_rd_addr + descr_offset];
end

initial begin
	$readmemh("usb3_descrip_rom.init", mem);
end



always @ ( * ) begin
	case ({wr_desc_act,wr_resp_act})
	2'b10:begin
		ep0_in_buf_data <= descr_data_q ;
	end
	2'b01:begin
		ep0_in_buf_data <= resp_data_q ;
	end
	default:begin
		ep0_in_buf_data <= 0 ;
	end
	endcase 
end   

assign resp_data_q = 32'hff_ff_ff_ff ;




// managing received data pkts

wire [31:0] ep0_out_buf_data_swap = swap32 ( ep0_out_buf_data ) ;

always @ ( posedge clk or negedge rst_n ) begin
	if (!rst_n) begin
		set_sel_act <= 0 ;
	end
	else if ( set_sel ) begin
		set_sel_act <= 1 ;
	end
	else if ( sync_frame ) begin
		sync_frame_act <= 1 ;
	end
	else if (  ep0_out_buf_data_val) begin
		case ( ep0_out_buf_data_inx ) 
		0:begin
			if ( set_sel_act ) begin
				u1_sel <= ep0_out_buf_data_swap[7:0];
				u1_pel <= ep0_out_buf_data_swap[15:8];
				u2_sel <= ep0_out_buf_data_swap[31:16];
			end
			else if ( sync_frame_act ) begin
				frame_num <= ep0_out_buf_data_swap[15:0] ;
				sync_frame_act <= 0 ;
			end
		end
		1:begin
			if ( set_sel_act ) begin 
				u2_pel <= ep0_out_buf_data_swap[31:0];
				set_sel_act <= 0 ;
			end
		end
		default:begin end
		endcase 
	end
end

always @ ( posedge clk or negedge rst_n ) begin
	if (!rst_n) begin
		rd_ep0_act <= 0 ;
	end
	else if ( rd_ep0_act ) begin
		if ( ep0_out_buf_data_ack ) begin
			rd_ep0_act <= 0 ;
		end
	end
	else if ( ep0_out_buf_hasdata ) begin
		rd_ep0_act <= 1 ;
		rd_ep0_len_latch <= ep0_out_buf_len ;
	end
	else begin
		rd_ep0_act <= 0 ;
	end
end

always @ ( posedge clk or negedge rst_n ) begin
	if (!rst_n) begin
		rd_ep0_pkt_act <= 0 ;
		rd_ep0_len_cnt <= 0 ;
	end	
	else if ( !rd_ep0_act_d && rd_ep0_act ) begin
		rd_ep0_pkt_act <= 1 ;
	end
	else if ( rd_ep0_pkt_act ) begin
		rd_ep0_len_cnt <= rd_ep0_len_cnt + 4 ;
		if ( rd_ep0_len_cnt + 4 >= rd_ep0_len_latch ) begin
			rd_ep0_pkt_act <= 0 ;
		end
	end
end

always @ ( posedge clk or negedge rst_n ) begin
	if (!rst_n) begin
		ep0_out_buf_rden <= 0 ; 
		ep0_out_buf_data_ack <= 0 ;
	end
	else if ( rd_ep0_pkt_act ) begin
		ep0_out_buf_rden <= 1 ;
	end
	else if ( rd_ep0_pkt_act_d && !rd_ep0_pkt_act ) begin
		ep0_out_buf_rden <= 0 ;
		ep0_out_buf_data_ack <= 1 ;
	end
	else begin
		ep0_out_buf_rden <= 0 ;
		ep0_out_buf_data_ack <= 0 ;	
	end
end

always @ ( posedge clk or negedge rst_n ) begin
	if (!rst_n) begin
		ep0_out_buf_data_inx <= 0 ;
	end
	else if ( ep0_out_buf_data_val ) begin
		ep0_out_buf_data_inx <= ep0_out_buf_data_inx + 1 ;
	end
	else begin
		ep0_out_buf_data_inx <= 0 ;
	end
end
	
	
	
	
	
	
function [31:0] swap32 ;

input [31:0] A  ;

	begin
		swap32 = {A[7:0],A[15:8],A[23:16],A[31:24]};
	end

endfunction


	
	
	
task show_time ;
	begin
		$display("at time %t ns\n",$realtime/100);
	end
endtask


endmodule 

