module DataTransfer
(
	 input wire	clk
	,input wire	rst_n

	//  ep1 IN 
	,output	reg		[31:0]	ep1_in_buf_data
	,output	reg				ep1_in_buf_wren
	,input	wire			ep1_in_buf_ready
	,output	reg				ep1_in_buf_data_commit
	,output	reg		[10:0]	ep1_in_buf_data_commit_len
	
	
	// ep2 OUT 
	,input	wire	[31:0]	ep1_out_buf_data
	,input	wire	[10:0]	ep1_out_buf_len
	,input	wire			ep1_out_buf_has_data
	,output reg 			ep1_out_buf_rden
	,output	reg				ep1_out_buf_data_ack 


);

reg rd_ep1_act_d ;
reg rd_ep1_pkt_act_d ;
reg rd_ep1_act ;
reg rd_ep1_pkt_act ;
reg [10:0] rd_ep1_pkt_len_latch ;
reg [10:0] rd_ep1_cnt ;
reg  ep1_out_buf_data_val ;
reg  ep1_out_buf_data_val_d ;

// do the loopback test 
always @ ( posedge clk or negedge rst_n ) begin
	if(!rst_n)begin
		rd_ep1_act_d <= 0 ;
		rd_ep1_pkt_act_d <= 0 ;
		ep1_out_buf_data_val_d <= 0 ;
	end
	else begin
		rd_ep1_act_d <= rd_ep1_act ;
		rd_ep1_pkt_act_d <= rd_ep1_pkt_act ;
		ep1_out_buf_data_val_d <= ep1_out_buf_data_val ;
	end
end


always @ ( posedge clk or negedge rst_n ) begin
	if(!rst_n)begin
		rd_ep1_act <= 0 ;
	end
	else if ( ep1_out_buf_data_ack ) begin
		rd_ep1_act <= 0 ;
	end
	else if ( ep1_out_buf_has_data ) begin
		rd_ep1_act <= 1 ;
	end
	else begin
		rd_ep1_act <= 0 ;
	end
end

always @ ( posedge clk or negedge rst_n ) begin
	if(!rst_n)begin
		rd_ep1_pkt_act <= 0 ;
	end
	else if ( !rd_ep1_act_d && rd_ep1_act ) begin
		rd_ep1_pkt_act <= 1 ;
		rd_ep1_pkt_len_latch <= ep1_out_buf_len ;
		rd_ep1_cnt <= 0 ;
	end
	else if ( rd_ep1_pkt_act ) begin
		rd_ep1_cnt <= rd_ep1_cnt + 4 ;
		if ( rd_ep1_cnt + 4 >= rd_ep1_pkt_len_latch ) begin
			rd_ep1_pkt_act <= 0 ;
		end
	end
end

always @ ( posedge clk or negedge rst_n ) begin
	if(!rst_n)begin
		ep1_out_buf_rden <= 0 ;
		ep1_out_buf_data_ack <= 0 ;
	end
	else if ( rd_ep1_pkt_act ) begin
		ep1_out_buf_rden <= 1 ;
	end
	else if ( rd_ep1_pkt_act_d && !rd_ep1_pkt_act ) begin
		ep1_out_buf_rden <= 0 ;
		ep1_out_buf_data_ack <= 1 ;
	end
	else begin
		ep1_out_buf_rden <= 0 ;
		ep1_out_buf_data_ack <= 0 ;	
	end
end

always @ ( posedge clk or negedge rst_n ) begin
	if(!rst_n)begin
		ep1_out_buf_data_val <= 0 ;
	end
	else begin
		ep1_out_buf_data_val <= ep1_out_buf_rden ;
	end
end
	
always @ ( posedge clk or negedge rst_n ) begin
	if(!rst_n)begin
		ep1_in_buf_wren <= 0 ;
		ep1_in_buf_data_commit <= 0 ;
	end	
	else if ( ep1_in_buf_ready && ep1_out_buf_data_val ) begin
		ep1_in_buf_wren <= 1 ;
		ep1_in_buf_data_commit <= 0 ;
		ep1_in_buf_data <= ep1_out_buf_data ;
		if ( ep1_in_buf_wren == 0 ) begin
			ep1_in_buf_data_commit_len <= ep1_out_buf_len ;
		end
	end
	else if ( ep1_out_buf_data_val_d && !ep1_out_buf_data_val ) begin
		ep1_in_buf_wren <= 0 ;
		ep1_in_buf_data_commit <= 1 ;
	end
	else begin
		ep1_in_buf_data_commit <= 0 ;
	end
end
	


endmodule