`timescale 1ns/1ns 
module usb3_host_protocol 
(
	input wire	clk
	,input wire rst_n 
	,input wire [4:0]	ltssm_state
	
	,input	wire				prot_rx_tp
	,input	wire				prot_rx_tp_hosterr
	,input	wire				prot_rx_tp_retry
	,input	wire				prot_rx_tp_pktpend
	,input	wire		[3:0]	prot_rx_tp_subtype
	,input	wire		[3:0]	prot_rx_tp_endp
	,input	wire		[4:0]	prot_rx_tp_nump
	,input	wire		[4:0]	prot_rx_tp_seq
	,input	wire		[15:0]	prot_rx_tp_stream
	
	,input	wire				prot_rx_dph
	,input	wire				prot_rx_dph_eob
	,input	wire				prot_rx_dph_setup
	,input	wire				prot_rx_dph_pktpend
	,input	wire		[3:0]	prot_rx_dph_endp
	,input	wire		[4:0]	prot_rx_dph_seq
	,input	wire		[15:0]	prot_rx_dph_len
	,input	wire				prot_rx_dpp_start
	,input	wire				prot_rx_dpp_done
	,input	wire				prot_rx_dpp_crcgood
	
	,output	reg				prot_tx_dph
	,output	reg				prot_tx_dph_eob
	,output	reg				prot_tx_dph_dir
	,output	reg	[3:0]		prot_tx_dph_endp
	,output	reg	[4:0]		prot_tx_dph_seq
	,output	reg	[15:0]		prot_tx_dph_len
	,output reg 			prot_tx_dph_setup 
	,input	wire			prot_tx_dpp_ack
	,input	wire			prot_tx_dpp_done	

	,output reg			prot_tx_tp_a
	,output reg			prot_tx_tp_a_retry
	,output reg			prot_tx_tp_a_dir
	,output reg	[3:0]	prot_tx_tp_a_subtype
	,output reg	[3:0]	prot_tx_tp_a_endp
	,output reg	[4:0]	prot_tx_tp_a_nump
	,output reg	[4:0]	prot_tx_tp_a_seq
	,output reg	[15:0]	prot_tx_tp_a_stream
	,output reg			prot_tx_tp_a_pp 
	,input	wire		prot_tx_tp_a_ack
	
	,input	wire	[9:0]	buf_out_addr
	,output	reg		[31:0]	buf_out_q
	,output	reg		[10:0]	buf_out_len
	,output	reg				buf_out_hasdata
	,input	wire			buf_out_arm
	,output	reg				buf_out_arm_ack	
	
	,input	wire		[9:0]	buf_in_addr
	,input	wire		[31:0]	buf_in_data
	,input	wire				buf_in_wren
	,output	reg					buf_in_ready
	,input	wire				buf_in_commit
	,input	wire		[10:0]	buf_in_commit_len
	,output	reg					buf_in_commit_ack
	

	,output reg 	[6:0]   dev_addr


);
initial begin
	show_time();
	$display("start link training\n");
	$display("this process takes 4.5ms\n");
	
	wait ( ltssm_state == 16 ) ;
	$display("link training completed \n");

end 


// 
parameter TRANSFER_NUM = 1024 ;



//tp subtype
parameter RSVD		= 4'b0000	;
parameter ACK		= 4'b0001	;
parameter NRDY		= 4'b0010	;
parameter ERDY		= 4'b0011	;
parameter STATUS	= 4'b0100	;
parameter STALL		= 4'b0101	;
parameter DEVNOTIFY	= 4'b0110	;
parameter PING		= 4'b0111	;
parameter PINGRSP	= 4'b1000	;



//bmRequestType
parameter BM_REQ_SET_ADDRESS 	= 8'b0000_0000 ;
parameter BM_REQ_GET_DESC 	 	= 8'b1000_0000 ;
parameter BM_REQ_SET_CONFIG  	= 8'b0000_0000 ;
parameter BM_REQ_SET_INTF    	= 8'b0000_0001 ;
parameter BM_REQ_GET_INTF    	= 8'b1000_0001 ;
parameter BM_REQ_SET_FEAT    	= 8'b0000_0001 ;
parameter BM_REQ_CLR_FEAT    	= 8'b0000_0001 ;
parameter BM_REQ_GET_DEV_STA    = 8'b1000_0000 ;
parameter BM_REQ_SET_SEL		= 8'b0000_0000 ;
parameter BM_REQ_GET_DEV_CONFIG	= 8'b1000_0000 ;

//bRequest
parameter SET_ADDRESS 			  = 8'd5 ; 
parameter GET_DESC 	  			  = 8'd6 ; 
parameter SET_CONFIGURATION 	  = 8'd9 ; 
parameter SET_INTERFACE 	 	  = 8'd11 ; 
parameter GET_INTERFACE 	 	  = 8'd10 ; 
parameter SET_FEATURE 	 	  	  = 8'd3 ; 
parameter CLR_FEATURE 	 	  	  = 8'd1 ; 
parameter GET_STA 	 	  	  	  = 8'd0 ; 
parameter SET_SEL				  = 8'd48 ;
parameter GET_CONFIGURATION		  = 8'd8 ;


//Desciptor types
parameter DEVICE 		= {8'd1,8'd0};
parameter CONFIGURATION = {8'd2,8'd0};
parameter BOS		 	= {8'd15,8'd0};


//device address 
parameter DEVICE_ADDRESS = 16'b0000_0000_0000_0001;


//host request 
parameter NO_REQ			   	 = 5'd0 ;
parameter GET_DEVICE_DESC 	   	 = 5'd1 ;
parameter SET_DEVICE_ADDRESS   	 = 5'd2 ;
parameter GET_FULL_CONFIG_DESC   = 5'd3 ;
parameter GET_PART_CONFIG_DESC   = 5'd4 ;
parameter GET_PART_BOS_DESC   	 = 5'd5 ;
parameter GET_FULL_BOS_DESC   	 = 5'd6 ;
parameter SET_DEVICE_CONFIG   	 = 5'd7 ;
parameter SET_DEV_INTF   	 	 = 5'd8 ;
parameter GET_DEV_INTF   	 	 = 5'd9 ;
parameter SET_DEV_FEAT   	 	 = 5'd10 ;
parameter CLR_DEV_FEAT   	 	 = 5'd11 ;
parameter GET_DEV_STA			 = 5'd12 ;
parameter SET_DEV_SEL			 = 5'd13 ;
parameter SET_UD_REQ			 = 5'd14 ;
parameter GET_DEVICE_CONFIG		 = 5'd15 ;

// host protocol state machine 
parameter HOST_STATE_W 					= 3;
parameter ST_HOST_IDLE 					= 3'd0;
parameter ST_HOST_WT_DONE 				= 3'd1;
parameter ST_HOST_BULK_OUT 				= 3'd2;
parameter ST_HOST_BULK_IN 				= 3'd3;

reg [HOST_STATE_W-1:0]	host_state ;

// pkts send state machine 
parameter SEND_STATE_W 		= 3;
parameter ST_SEND_IDLE 		= 3'd0;
parameter ST_SEND_DPH 		= 3'd1;
parameter ST_SEND_DPP 		= 3'd2;
parameter ST_SEND_TP 		= 3'd3;

reg [SEND_STATE_W-1:0]	send_state ;

// transfer state 
parameter TRANSF_STATE_W 	 = 3;
parameter ST_TRANSF_IDLE 	 = 3'd0;
parameter ST_TRANSF_BULK_OUT = 3'd1;
parameter ST_TRANSF_BULK_IN  = 3'd2;

reg	[TRANSF_STATE_W-1:0] trasf_state ;

// control transfer state machine 
parameter CTRL_STATE_W 			= 3;
parameter ST_CTRL_IDLE 			= 3'd0;
parameter ST_CTRL_SEND_DP 		= 3'd1;
parameter ST_CTRL_WT_RESP 		= 3'd2;
parameter ST_CTRL_SEND_TP		= 3'd3;

reg [CTRL_STATE_W-1:0]	ctrl_state ;

// pkts receive state machine
parameter RECV_STATE_W 		= 3;
parameter ST_RECV_IDLE 		= 3'd0;

reg [RECV_STATE_W-1:0]	recv_state ;


// dpp parsing state machine
parameter RD_DPP_STATE_W 	= 3;
parameter ST_RD_DPP_IDLE 	= 3'd0;
parameter ST_RD_DPP_ACTIVE 	= 3'd1;

reg [RD_DPP_STATE_W-1:0] rd_dpp_state ;

reg load_request 		;
reg [7:0]bmRequestType 	;	
reg [7:0]bRequest       ;
reg [15:0]wValue        ;
reg [15:0]wIndex	    ;
reg [15:0]wLength		;
reg [3:0]rx_tp_subtype	;
reg [3:0]rx_tp_endp	    ;
reg [4:0]rx_tp_seq      ;
reg      rx_tp_active   ;
reg  	 expect_ack     ;
reg 	 tx_dph_eob		;
reg 	 tx_dph_dir	    ;
reg [3:0]tx_dph_endp    ;
reg [4:0]tx_dph_seq	    ;
reg [15:0]tx_dph_len	;
reg		 tx_dph 		;
reg 	 device_attached;
reg      device_addressed;
reg 	 data_in_stage_pending ;
reg 	 data_out_stage_pending ;
reg 	 tx_tp_a ;
reg		 tx_tp_a_retry ;
reg		 tx_tp_a_dir ;
reg	[3:0]tx_tp_a_subtype ;
reg [3:0]tx_tp_a_endp;
reg [4:0]tx_tp_a_nump;
reg [4:0]tx_tp_a_seq;
reg [15:0] tx_tp_a_stream ;
reg		 setup_stage ;
reg		 data_in_stage ;
reg		 status_stage ;
reg tx_dph_setup ;
reg [4:0] dp_seq ;
reg				rx_dph			;
reg				rx_dph_eob      ;
reg				rx_dph_setup    ;
reg				rx_dph_pktpend  ;
reg		[3:0]	rx_dph_endp     ;
reg		[4:0]	rx_dph_seq      ;
reg		[15:0]	rx_dph_len      ;
reg				rx_dpp_start    ;
reg				rx_dpp_done     ;
reg				rx_dpp_crcgood  ;
reg				got_part_device_desc;
reg		[3:0]   request ;
reg				got_full_device_desc ;
reg				expect_tp ;
reg		[4:0]   expect_dp_num ;
reg 			rx_dph_active ;
reg				tx_tp_a_pp ;
reg		[3:0]	expect_tp_type ;
reg		[4:0]	expect_tp_seq ;
reg				rx_tp_hosterr ;
reg				rx_tp_pktpend ;
reg 	[10:0]  dpp_len ;
reg 		    rx_dpp_active ;
reg		[4:0]	expect_dp_seq ;
reg		[7:0]	rd_dpp_addr ;
reg				part_device_desc_update ;
reg		[7:0]	bLength ;	
reg		[7:0]	bDescriptorType ;	
reg		[15:0]	bcdUSB ;
reg		[31:0]  dpp_data ;
reg				got_device_desc ;
reg				device_desc_update ;
reg				got_full_config_desc ;
reg				got_part_config_desc ;
reg				got_full_bos_desc ;
reg				device_configured ;
reg				config_desc_update ;
reg				bos_desc_update ;
reg [15:0]		wTotalLength ;
reg [15:0]		config_total_len ;
reg [15:0]		bos_total_len ;
reg				got_part_bos_desc ;
reg	[4:0]		tx_ack_seq ;
reg	[4:0]		ack_tp_seq ;
reg	[4:0]		ack_dp_seq ;
reg	[4:0]		out_seq ;
reg				out_en ;
reg			 	transfer_active ;
reg				go_on ;
reg				set_interface_setting ;
reg				get_interface_setting ;
reg				alt_setting_update ;
reg	[7:0]		alt_setting ;
reg				set_feat ;
reg				clr_feat ;
reg				got_dev_status ;
reg [4:0]		ep2_acked_dp_seq ;
reg	[4:0]		ep2_seq_num ;
reg	[4:0]		ep1_seq_num ;
reg	[4:0]		ep0_seq_num ;
reg [15:0]		ep2_transfer_cnt ;
reg [15:0]		ep1_transfer_cnt ;
reg 			in_end ;
reg				set_sel;
reg				data_out_stage;
reg				set_undefine_req ;
reg 			get_dev_config ;
reg [4:0] out_endpt_seq_num ;
reg [4:0] in_endpt_seq_num ;
reg [15:0] tx_pkt_cnt ; 
reg [15:0] rx_pkt_cnt ; 
reg [3:0] endpt_num ;
reg rx_dpp_act;
reg rx_dpp_act_end = 0 ;
reg tx_pkt_act ;
reg rx_pkt_act = 0 ;
reg loopback = 0 ;

initial begin
	wait ( ltssm_state == 5'd16 ) ;
	#2000
	device_attached <= 1 ;
end

wire [4:0] ack_dp_seq_inc = ack_dp_seq + 1 ;

always @ ( posedge clk or negedge rst_n ) begin
	if (!rst_n) begin
		host_state <= ST_HOST_IDLE ;
		send_state <= ST_SEND_IDLE ;
		ctrl_state <= ST_CTRL_IDLE ;
		recv_state <= ST_RECV_IDLE ;
		rd_dpp_state <= ST_RD_DPP_IDLE ;
		trasf_state <= ST_TRANSF_IDLE ;
		
		device_addressed <= 0 ;
		
		prot_tx_dph 		<= 0 ; 	  	
		prot_tx_dph_eob		<= 0 ; 
		prot_tx_dph_dir		<= 0 ; 
		prot_tx_dph_endp	<= 0 ; 
		prot_tx_dph_seq		<= 0 ; 
		prot_tx_dph_len		<= 0 ; 
		prot_tx_dph_setup   <= 0 ;
		
		load_request 	<= 0 ;
		bmRequestType	<= 0 ;	
		bRequest        <= 0 ;
		wValue          <= 0 ;
		wIndex	        <= 0 ;
		wLength			<= 0 ;	

		tx_dph 			<= 0 ;
		tx_dph_eob		<= 0 ;
		tx_dph_dir	    <= 0 ;
		tx_dph_endp     <= 0 ;
		tx_dph_seq	    <= 0 ;
		tx_dph_len      <= 0 ;
		tx_dph_setup    <= 0 ;
			
		rx_tp_subtype	<= 0 ;	
		rx_tp_endp	    <= 0 ;
		rx_tp_seq       <= 0 ;
		rx_tp_active   	<= 0 ;	
		

		data_in_stage_pending <= 0 ;
		data_out_stage_pending <= 0 ;		
		device_attached <= 0 ;
		got_device_desc <= 0 ;
		got_full_config_desc <= 0 ;
		got_part_config_desc <= 1 ;
		got_part_bos_desc <= 1 ;
		got_full_bos_desc <= 0 ;
		dev_addr <= 0 ;
		device_configured <= 0 ;
		out_seq <= 0 ;
		tx_ack_seq <= 0 ;
		out_en <= 0 ;
		transfer_active <= 0 ;
		set_interface_setting <= 0 ;
		get_interface_setting <= 0 ;
		set_feat <= 0 ;
		clr_feat <= 0 ;
		got_dev_status <= 0 ;
		ep2_seq_num <= 0 ;
		ep1_seq_num <= 0 ;
		ep2_transfer_cnt <= 0 ;
		ep2_acked_dp_seq <= 0 ;
		ep1_transfer_cnt <= 0 ;
		in_end <= 0 ;
		set_sel <= 0 ;
		ep0_seq_num <= 0 ;
		set_undefine_req <= 0 ;
		get_dev_config <= 0 ;
	end
	
	
	// host protocol 
	// device attached >> get device descriptor >>  get full device descriptor >> assign new address >> 
	// get configuration descriptor >> get full configuration descriptor >> 
	// set configuration >> set interface 

	case ( host_state )
	ST_HOST_IDLE:begin	//0
		if ( device_attached  && !transfer_active ) begin
			if ( !got_device_desc ) begin 
				host_state <= ST_HOST_WT_DONE ;
				ctrl_state <= ST_CTRL_SEND_DP ;
				setup_stage <= 1 ;		
				request <= GET_DEVICE_DESC  ;		
			end 
			else if ( !device_addressed ) begin
				host_state <= ST_HOST_WT_DONE ;
				ctrl_state <= ST_CTRL_SEND_DP ;
				setup_stage <= 1 ;		
				request <= SET_DEVICE_ADDRESS  ;
			end
			else if ( !got_part_config_desc ) begin
				host_state <= ST_HOST_WT_DONE ;
				ctrl_state <= ST_CTRL_SEND_DP ;
				setup_stage <= 1 ;		
				request <= GET_PART_CONFIG_DESC  ;				
			end
			else if ( !got_full_config_desc ) begin
				host_state <= ST_HOST_WT_DONE ;
				ctrl_state <= ST_CTRL_SEND_DP ;
				setup_stage <= 1 ;		
				request <= GET_FULL_CONFIG_DESC  ;			
			end
			else if ( !got_part_bos_desc ) begin
				host_state <= ST_HOST_WT_DONE ;
				ctrl_state <= ST_CTRL_SEND_DP ;
				setup_stage <= 1 ;		
				request <= GET_PART_BOS_DESC  ;				
			end
			else if ( !got_full_bos_desc ) begin
				host_state <= ST_HOST_WT_DONE ;
				ctrl_state <= ST_CTRL_SEND_DP ;
				setup_stage <= 1 ;		
				request <= GET_FULL_BOS_DESC  ;				
			end
			else if ( !device_configured ) begin
				host_state <= ST_HOST_WT_DONE ;
				ctrl_state <= ST_CTRL_SEND_DP ;
				setup_stage <= 1 ;		
				request <= SET_DEVICE_CONFIG  ;					
			end
			else if ( !get_dev_config ) begin
				host_state <= ST_HOST_WT_DONE ;
				ctrl_state <= ST_CTRL_SEND_DP ;
				setup_stage <= 1 ;		
				request <= GET_DEVICE_CONFIG  ;						
			end
			else if ( !set_interface_setting ) begin
				host_state <= ST_HOST_WT_DONE ;
				ctrl_state <= ST_CTRL_SEND_DP ;
				setup_stage <= 1 ;		
				request <= SET_DEV_INTF  ;					
			end
			else if ( !get_interface_setting ) begin
				host_state <= ST_HOST_WT_DONE ;
				ctrl_state <= ST_CTRL_SEND_DP ;
				setup_stage <= 1 ;		
				request <= GET_DEV_INTF  ;				
			end
			else if ( !set_feat ) begin
				host_state <= ST_HOST_WT_DONE ;
				ctrl_state <= ST_CTRL_SEND_DP ;
				setup_stage <= 1 ;		
				request <= SET_DEV_FEAT  ;				
			end
			else if ( !clr_feat ) begin
				host_state <= ST_HOST_WT_DONE ;
				ctrl_state <= ST_CTRL_SEND_DP ;
				setup_stage <= 1 ;		
				request <= CLR_DEV_FEAT  ;			
			end
			else if ( !set_sel ) begin
				host_state <= ST_HOST_WT_DONE ;
				ctrl_state <= ST_CTRL_SEND_DP ;
				setup_stage <= 1 ;
				request <= SET_DEV_SEL ;
			end	
			//else if ( !set_undefine_req ) begin
			//	host_state <= ST_HOST_WT_DONE ;
			//	ctrl_state <= ST_CTRL_SEND_DP ;
			//	setup_stage <= 1 ;
			//	request <= SET_UD_REQ ;				
			//end			
			else if ( !got_dev_status ) begin
				host_state <= ST_HOST_WT_DONE ;
				ctrl_state <= ST_CTRL_SEND_DP ;
				setup_stage <= 1 ;
				request <= GET_DEV_STA ;
			end	
		end 	
		else if ( transfer_active ) begin
			host_state <= ST_HOST_BULK_OUT ;
			ctrl_state <= ST_CTRL_SEND_DP ;			
		end
	end
	ST_HOST_WT_DONE:begin //1
		if ( ctrl_state == ST_CTRL_IDLE ) begin
			host_state <= ST_HOST_IDLE ;			
			if ( request == GET_DEVICE_DESC  ) begin
				got_device_desc <= 1 ; 
			end
			else if ( request == SET_DEVICE_ADDRESS  ) begin
				device_addressed <= 1 ;			
			end	
			else if ( request == GET_PART_CONFIG_DESC  ) begin
				got_part_config_desc <= 1 ;				
			end			
			else if ( request == GET_FULL_CONFIG_DESC  ) begin
				got_full_config_desc <= 1 ;				
			end
			else if ( request == GET_PART_BOS_DESC  ) begin
				got_part_bos_desc <= 1 ;				
			end
			else if ( request == GET_FULL_BOS_DESC  ) begin
				got_full_bos_desc <= 1 ;				
			end
			else if ( request == SET_DEVICE_CONFIG  ) begin
				device_configured <= 1 ;				
			end
			else if ( request == GET_DEVICE_CONFIG ) begin
				get_dev_config <= 1 ;
			end
			else if ( request == SET_DEV_INTF  ) begin
				set_interface_setting <= 1 ;				
			end
			else if ( request == GET_DEV_INTF  ) begin
				get_interface_setting <= 1 ;				
			end
			else if ( request == SET_DEV_FEAT ) begin
				set_feat <= 1 ;				
			end
			else if ( request == CLR_DEV_FEAT ) begin
				clr_feat <= 1 ;				
			end
			else if ( request == SET_DEV_SEL ) begin
				set_sel <= 1 ;		
			end
			else if ( request == SET_UD_REQ ) begin
				set_undefine_req <= 1 ;
			end
			else if ( request == GET_DEV_STA ) begin
				got_dev_status <= 1 ;									
				transfer_active <= 0 ;

				show_time();
				$display("device passed enumeration\n");
				
				//loopback <= 1 ;
				//loopback_test() ;
				
				
				$finish();
			end	
		end
	end
	ST_HOST_BULK_OUT : begin
		if ( ctrl_state == ST_CTRL_IDLE ) begin		
			host_state <= ST_HOST_BULK_IN ;
			ctrl_state <= ST_CTRL_SEND_TP ;
		end
	end
	ST_HOST_BULK_IN: begin
		if ( ctrl_state == ST_CTRL_IDLE && send_state == ST_SEND_IDLE ) begin
			host_state <= ST_HOST_BULK_OUT ;
			ctrl_state <= ST_CTRL_SEND_DP ;			
		end
	end
	
	
	endcase
	
	load_request <= 0 ;
	
	case ( ctrl_state ) 
	ST_CTRL_IDLE:begin //0
	end
	ST_CTRL_SEND_DP:begin//1
		if ( setup_stage ) begin
			send_state <= ST_SEND_DPH ;			
			ctrl_state <= ST_CTRL_WT_RESP ;	
		end
		else if ( data_out_stage ) begin
			send_state <= ST_SEND_DPH ;			
			ctrl_state <= ST_CTRL_WT_RESP ;			
		end
		else if ( transfer_active ) begin
			send_state <= ST_SEND_DPH ;	
			ctrl_state <= ST_CTRL_WT_RESP ;				
		end
		if ( setup_stage && request == SET_DEVICE_ADDRESS ) begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;		
			data_in_stage_pending <= 0 ;
			data_out_stage_pending <= 0 ;
			bmRequestType	<= BM_REQ_SET_ADDRESS;	
			bRequest        <= SET_ADDRESS;
			wValue          <= DEVICE_ADDRESS;
			wIndex	        <= 16'b0 ;
			wLength			<= 16'd8 ;
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
		end
		else if ( setup_stage && request == GET_DEVICE_DESC  ) begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;
			data_in_stage_pending <= 1 ;
			data_out_stage_pending <= 0 ;			
			bmRequestType	<= BM_REQ_GET_DESC;	
			bRequest        <= GET_DESC;
			wValue          <= DEVICE  ;
			wIndex	        <= 16'b0 ;
			wLength			<= 16'd18 ;	
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
			expect_dp_num <= 1 ;
		end
		else if ( setup_stage && request == GET_PART_CONFIG_DESC ) begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;
			data_in_stage_pending <= 1 ;
			data_out_stage_pending <= 0 ;			
			bmRequestType	<= BM_REQ_GET_DESC;	
			bRequest        <= GET_DESC;
			wValue          <= CONFIGURATION  ;
			wIndex	        <= 16'b0 ;
			wLength			<= 16'd9 ;	
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
			expect_dp_num <= 1 ;			
		end
		else if ( setup_stage && request == GET_FULL_CONFIG_DESC ) begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;
			data_in_stage_pending <= 1 ;
			data_out_stage_pending <= 0 ;			
			bmRequestType	<= BM_REQ_GET_DESC;	
			bRequest        <= GET_DESC;
			wValue          <= CONFIGURATION  ;
			wIndex	        <= 16'b0 ;
			wLength			<= 256 ;	
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
			expect_dp_num <= 1 ;			
		end
		else if ( setup_stage && request == GET_PART_BOS_DESC ) begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;
			data_in_stage_pending <= 1 ;
			data_out_stage_pending <= 0 ;			
			bmRequestType	<= BM_REQ_GET_DESC;	
			bRequest        <= GET_DESC;
			wValue          <= BOS  ;
			wIndex	        <= 16'b0 ;
			wLength			<= 5 ;	
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
			expect_dp_num <= 1 ;			
		end
		else if ( setup_stage && request == GET_DEVICE_CONFIG ) begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;
			data_in_stage_pending <= 1 ;
			data_out_stage_pending <= 0 ;			
			bmRequestType	<= BM_REQ_GET_DEV_CONFIG;	
			bRequest        <= GET_CONFIGURATION;
			wValue          <= 0  ;
			wIndex	        <= 16'b0 ;
			wLength			<= 1 ;	
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
			expect_dp_num <= 1 ;		
		end
		else if ( setup_stage && request == GET_FULL_BOS_DESC ) begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;
			data_in_stage_pending <= 1 ;
			data_out_stage_pending <= 0 ;			
			bmRequestType	<= BM_REQ_GET_DESC;	
			bRequest        <= GET_DESC;
			wValue          <= BOS  ;
			wIndex	        <= 16'b0 ;
			wLength			<= 256 ;	
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
			expect_dp_num <= 1 ;			
		end
		else if ( setup_stage && request == SET_DEVICE_CONFIG ) begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;
			data_in_stage_pending <= 0 ;
			data_out_stage_pending <= 0 ;			
			bmRequestType	<= BM_REQ_SET_CONFIG;	
			bRequest        <= SET_CONFIGURATION;
			wValue          <= 1  ;
			wIndex	        <= 16'b0 ;
			wLength			<= 0 ;	
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
			expect_dp_num <= 0 ;			
		end
		else if ( setup_stage && request == SET_DEV_INTF ) begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;
			data_in_stage_pending <= 0 ;
			data_out_stage_pending <= 0 ;			
			bmRequestType	<= BM_REQ_SET_INTF;	
			bRequest        <= SET_INTERFACE;
			wValue          <= 16'd1 ;
			wIndex	        <= 16'd1 ;
			wLength			<= 16'd0 ;	
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
			expect_dp_num <= 0 ;		
		end
		else if ( setup_stage && request == GET_DEV_INTF ) begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;
			data_in_stage_pending <= 1 ;
			data_out_stage_pending <= 0 ;			
			bmRequestType	<= BM_REQ_GET_INTF;	
			bRequest        <= GET_INTERFACE;
			wValue          <= 16'd0 ;
			wIndex	        <= 16'd1 ;
			wLength			<= 16'd1 ;	
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
			expect_dp_num <= 1 ;		
		end
		else if ( setup_stage && request == SET_DEV_FEAT )  begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;
			data_in_stage_pending <= 0 ;
			data_out_stage_pending <= 0 ;			
			bmRequestType	<= BM_REQ_SET_FEAT;	
			bRequest        <= SET_FEATURE;
			wValue          <= 16'd1 ;
			wIndex	        <= 16'd1 ;
			wLength			<= 16'd0 ;	
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
			expect_dp_num <= 0 ;		
		end
		else if ( setup_stage && request == SET_DEV_SEL ) begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;
			data_in_stage_pending <= 0 ;
			data_out_stage_pending <= 1 ;	
			ep0_seq_num		<= 0 ;		
			bmRequestType	<= BM_REQ_SET_SEL;	
			bRequest        <= SET_SEL;
			wValue          <= 16'd0 ;
			wIndex	        <= 16'd0 ;
			wLength			<= 16'd6 ;	
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
			expect_dp_num <= 0 ;	
			
		end
		else if ( setup_stage && request == CLR_DEV_FEAT )  begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;
			data_in_stage_pending <= 0 ;
			data_out_stage_pending <= 0 ;			
			bmRequestType	<= BM_REQ_CLR_FEAT;	
			bRequest        <= CLR_FEATURE;
			wValue          <= 16'd1 ;
			wIndex	        <= 16'd1 ;
			wLength			<= 16'd0 ;	
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
			expect_dp_num <= 0 ;		
		end
		else if ( setup_stage && request == GET_DEV_STA  ) begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;
			data_in_stage_pending <= 1 ;
			data_out_stage_pending <= 0 ;			
			bmRequestType	<= BM_REQ_GET_DEV_STA;	
			bRequest        <= GET_STA;
			wValue          <= 16'd0 ;
			wIndex	        <= 16'd0 ;
			wLength			<= 16'd2 ;	
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
			expect_dp_num <= 1 ;
		end
		else if ( setup_stage && request == SET_UD_REQ ) begin
			load_request 	<= 1 ;	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len		<= 8 ;
			tx_dph_setup    <= 1 ;
			dp_seq			<= 0 ;
			data_in_stage_pending <= 0 ;
			data_out_stage_pending <= 1 ;			
			bmRequestType	<= 0 ;	
			bRequest        <= 0 ;
			wValue          <= 16'd0 ;
			wIndex	        <= 16'd0 ;
			wLength			<= 16'd1 ;	
			expect_tp     <= 1 ;
			expect_tp_seq <= 1 ;
			expect_dp_num <= 0 ;		
		end
		else if ( transfer_active ) begin	
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 2 ; 
			tx_dph_seq		<= ep2_seq_num ; 
			tx_dph_len      <= ep2_transfer_cnt[15:0]  ;
			tx_dph_setup    <= 0 ;
		
			ep2_seq_num <= ep2_seq_num + 1 ;			

		end
		else if ( data_out_stage ) begin
			tx_dph			<= 1 ;
			tx_dph_eob		<= 0 ; 
			tx_dph_dir		<= 0 ; 
			tx_dph_endp		<= 0 ; 
			tx_dph_seq		<= 0 ; 
			tx_dph_len      <= wLength ;
			tx_dph_setup    <= 0 ;	

			ep0_seq_num <= ep0_seq_num + 1 ;
			
		end
	end 
	ST_CTRL_WT_RESP:begin//2
		if ( setup_stage  ) begin
			if (  rx_tp_active && rx_tp_subtype == ACK && rx_tp_endp == 0 && rx_tp_seq == expect_tp_seq  ) begin
				setup_stage <= 0 ;
				if ( data_in_stage_pending ) begin
					ctrl_state  <= ST_CTRL_SEND_TP ;
					data_in_stage <= 1 ;
				end
				else if ( data_out_stage_pending ) begin
					ctrl_state <= ST_CTRL_SEND_DP ;
					data_out_stage <= 1 ;
				end
				else begin
					ctrl_state <= ST_CTRL_SEND_TP ;
					status_stage <= 1 ;
				end
			end
		end
		else if ( data_out_stage ) begin
			if  ( rx_tp_active && rx_tp_subtype == ACK && rx_tp_endp == 0 && rx_tp_seq == expect_tp_seq ) begin
				data_out_stage <= 0 ;
				ctrl_state <= ST_CTRL_SEND_TP ;
				status_stage <= 1  ;
			end
		end
		else if ( data_in_stage  ) begin
			if ( rx_dph_active ) begin
				if ( rx_dph_endp == 0 && rx_dph_seq == expect_dp_seq ) begin
				end 
			end
			else if ( rx_dpp_active ) begin
				expect_dp_num <= expect_dp_num - 1 ;
				if ( expect_dp_num == 1 ) begin
					data_in_stage <= 0 ;
					status_stage <= 1 ;
					ctrl_state  <= ST_CTRL_SEND_TP ;
				end				
			end
		end
		else if ( status_stage  ) begin
			if (  rx_tp_active && rx_tp_subtype == ACK && rx_tp_endp == 0 && rx_tp_seq == expect_tp_seq  ) begin
				status_stage <= 0 ;
				ctrl_state  <= ST_CTRL_IDLE ;
			end
		end
		else if ( transfer_active ) begin
			if (  rx_tp_active && rx_tp_subtype == ACK && rx_tp_endp == 2 && rx_tp_seq == ep2_seq_num  ) begin
				ctrl_state  <= ST_CTRL_IDLE ;
				ep2_transfer_cnt <= ep2_transfer_cnt + 1 ;
				if ( ep2_transfer_cnt == 1024 ) begin
					ep2_transfer_cnt <=  0 ;
				end
			end
			else if ( rx_tp_active && rx_tp_subtype == NRDY && rx_tp_endp == 2 ) begin
				ctrl_state  <= ST_CTRL_IDLE ;			
			end
			else if ( rx_tp_active && rx_tp_subtype == ERDY && rx_tp_endp == 1 ) begin
				ctrl_state  <= ST_CTRL_IDLE ;			
			end			
			else if ( rx_dpp_active ) begin
				ctrl_state  <= ST_CTRL_SEND_TP ;
				ep1_seq_num <= ep1_seq_num + 1 ;
				in_end <= 1 ;
				ep1_transfer_cnt <= ep1_transfer_cnt + 1 ;
				if ( dpp_len == 1024 ) begin
					$finish ;
				end
				
				if ( dpp_len == ep1_transfer_cnt  ) begin
					$display(" loopback transfer %d succeed  \n " , ep1_transfer_cnt );
				end
				else begin
					$display(" loopback transfer %d error  \n " , ep1_transfer_cnt );				
					$finish ;
				end

			end
		end
	end
	ST_CTRL_SEND_TP:begin //3
		if ( host_state == ST_HOST_WT_DONE ) begin
			tx_tp_a			<= 1 ;	
			tx_tp_a_retry	<= 0 ;	
			tx_tp_a_dir		<= 0 ;
			tx_tp_a_endp	<= 0 ;	
			dev_addr	 	<= ( device_addressed ) ? DEVICE_ADDRESS[6:0] : 7'b0  ;		
			tx_tp_a_stream  <= 0 ;	
			if ( status_stage ) begin
				tx_tp_a_subtype <= STATUS ;
				tx_tp_a_nump    <= expect_dp_num ;
				tx_tp_a_seq     <= 0 ;
				tx_tp_a_pp		<= 0 ;
				expect_tp_seq   <= 1 ;
				expect_tp_type  <= ACK ;
				expect_tp		<= 1 ;
			end
			else if ( data_in_stage ) begin
				tx_tp_a_subtype <= ACK ;
				tx_tp_a_nump    <= expect_dp_num ;
				tx_tp_a_seq     <= dp_seq ;
				tx_tp_a_pp		<= 1 ;
				expect_dp_seq	<= dp_seq ;
			end
			ctrl_state		<= ST_CTRL_WT_RESP ;
			send_state <= ST_SEND_TP ;
		end
		else if ( transfer_active  ) begin
			tx_tp_a			<= 1 ;	
			tx_tp_a_retry	<= 0 ;	
			tx_tp_a_dir		<= 1 ;
			tx_tp_a_endp	<= 1 ;	
			dev_addr	 	<= ( device_addressed ) ? DEVICE_ADDRESS[6:0] : 7'b0  ;		
			tx_tp_a_stream  <= 0 ;	
			tx_tp_a_subtype <= ACK ;
			if ( in_end ) begin
				in_end <= 0 ;
				tx_tp_a_nump    <= 0 ;
				ctrl_state		<= ST_CTRL_IDLE ;				
			end
			else begin
				tx_tp_a_nump    <= 1 ; 
				ctrl_state		<= ST_CTRL_WT_RESP ;				
			end
			tx_tp_a_seq     <= ep1_seq_num ;
			tx_tp_a_pp		<= 1 ;
			
			send_state 		<= ST_SEND_TP ;		
		end
	end
	endcase 
	



	if ( transfer_active ) begin
		if ( rx_tp_active &&  rx_tp_endp == 2 && rx_tp_subtype == ACK  ) begin
			if (  rx_tp_seq  == ack_dp_seq_inc) begin
				ack_dp_seq <= ack_dp_seq + 1  ;
			end
		end	
	end
	
	if ( ack_dp_seq <= out_seq ) begin
		if ( out_seq - ack_dp_seq < 3 ) begin
			go_on <= 1 ;
		end
		else begin
			go_on <= 0 ;
		end
	end
	else if ( {1'b1,out_seq[3:0]} - {1'b0,ack_dp_seq[3:0]} < 3  ) begin
		go_on <= 1 ;
	end
	else begin
		go_on <= 0 ;
	end
	
	case ( trasf_state ) 
	ST_TRANSF_IDLE:begin
		if ( transfer_active ) begin
			dp_seq <= 0 ;
			ack_dp_seq <= 0 ;
			`ifdef BULK 
				trasf_state <= ST_TRANSF_BULK_OUT ;
			`endif 
		end
	end
	ST_TRANSF_BULK_OUT:begin
		if ( go_on && send_state == ST_SEND_IDLE) begin 
			send_state 		<= ST_SEND_DPH ;
			tx_dph 			<= 1		   ;
			tx_dph_eob	    <= 0		   ;
			tx_dph_dir	    <= 0		   ;
			tx_dph_endp 	<= 2		   ;
			tx_dph_seq	    <= out_seq	   ;
			tx_dph_len	    <= 512		   ;
			tx_dph_setup    <= 0		   ;
			out_en <= 1 ;	
		end
		else if ( buf_out_arm ) begin
			out_seq	<= out_seq + 1 ;
			trasf_state <= ST_TRANSF_BULK_IN  ;
		end		
	end
	ST_TRANSF_BULK_IN:begin
		if ( send_state == ST_SEND_IDLE ) begin
			tx_tp_a			<= 1 ;	
			tx_tp_a_retry	<= 0 ;	
			tx_tp_a_dir		<= 1 ;
			tx_tp_a_endp	<= 1 ;	
			dev_addr	 	<= ( device_addressed ) ? DEVICE_ADDRESS[6:0] : 7'b0  ;		
			tx_tp_a_stream  <= 0 ;		
			tx_tp_a_subtype <= ACK ;
			tx_tp_a_nump    <= 3 ;
			tx_tp_a_seq     <= dp_seq ;
			tx_tp_a_pp		<= 1 ;
			expect_dp_seq	<= dp_seq ;	
			send_state <= ST_SEND_TP ;
			trasf_state <= ST_TRANSF_BULK_OUT ;
		end
	end
	endcase

	case ( send_state ) 
	ST_SEND_IDLE:begin //0
		prot_tx_tp_a <= 0 ;
	end
	ST_SEND_DPH:begin //1
		prot_tx_dph 	 	<= tx_dph 		 ; 
		prot_tx_dph_eob	 	<= tx_dph_eob	 ; 
		prot_tx_dph_dir	 	<= tx_dph_dir	 ; 
		prot_tx_dph_endp 	<= tx_dph_endp 	 ; 
		prot_tx_dph_seq	 	<= tx_dph_seq	 ; 
		prot_tx_dph_len	 	<= tx_dph_len	 ;
		prot_tx_dph_setup	<= tx_dph_setup  ;
		if ( prot_tx_dpp_ack ) begin
			prot_tx_dph <= 0 ;
			send_state <= ST_SEND_DPP ;
		end			
	end
	ST_SEND_DPP:begin //2
		if ( buf_out_arm ) begin
			send_state <= ST_SEND_IDLE ;	
		end	
	end
	ST_SEND_TP:begin //3
		prot_tx_tp_a		 <= tx_tp_a			;
		prot_tx_tp_a_retry	 <= tx_tp_a_retry	;
		prot_tx_tp_a_dir	 <= tx_tp_a_dir		;
		prot_tx_tp_a_subtype <= tx_tp_a_subtype ;
		prot_tx_tp_a_endp	 <= tx_tp_a_endp	;
		prot_tx_tp_a_nump    <= tx_tp_a_nump    ;
		prot_tx_tp_a_seq	 <= tx_tp_a_seq		;
		prot_tx_tp_a_stream  <= tx_tp_a_stream  ;
		prot_tx_tp_a_pp		 <= tx_tp_a_pp		;
		if ( prot_tx_tp_a_ack ) begin
			prot_tx_tp_a <= 0 ;
			send_state <= ST_SEND_IDLE ;
		end
	end
	endcase 
	
	case ( recv_state ) 
	ST_RECV_IDLE:begin //0
		rx_tp_active <= 0 ;
		rx_dph_active <= 0 ;
		rx_dpp_active <= 0 ;
		if ( prot_rx_tp ) begin
			rx_tp_subtype <= prot_rx_tp_subtype ;
			rx_tp_endp	  <= prot_rx_tp_endp    ;
			rx_tp_seq     <= prot_rx_tp_seq     ;
			rx_tp_hosterr <= prot_rx_tp_hosterr ;
			rx_tp_pktpend <= prot_rx_tp_pktpend ;
			rx_tp_active  <= 1 ;
		end	 
		else if ( prot_rx_dph ) begin
			rx_dph_eob		<= 	prot_rx_dph_eob		;
			rx_dph_setup	<= 	prot_rx_dph_setup   ;
			rx_dph_pktpend	<= 	prot_rx_dph_pktpend ;
			rx_dph_endp		<= 	prot_rx_dph_endp    ;
			rx_dph_seq		<= 	prot_rx_dph_seq     ;
			rx_dph_len		<= 	prot_rx_dph_len     ;
			rx_dpp_start	<= 	prot_rx_dpp_start   ;
			rx_dpp_done		<= 	prot_rx_dpp_done    ;
			rx_dpp_crcgood	<= 	prot_rx_dpp_crcgood ;
			rx_dph_active   <= 1 ;
		end
		else if ( buf_in_commit ) begin
			rx_dpp_active <= 1 ;
			dpp_len <= buf_in_commit_len ;
			rd_dpp_state <= ST_RD_DPP_ACTIVE ;
		end
	end
	endcase

	case ( rd_dpp_state ) 
	ST_RD_DPP_IDLE:begin
		rd_dpp_addr <= 0 ;
		device_desc_update <= 0 ;
		config_desc_update <= 0 ;
		alt_setting_update <= 0 ;
		buf_in_ready <= 1 ;
	end
	ST_RD_DPP_ACTIVE:begin
		buf_in_ready <= 0 ;	
		rd_dpp_addr <= rd_dpp_addr + 1 ;
		part_device_desc_update <= 0 ;
		if ( rd_dpp_addr >= dpp_len[10:2] ) begin
			rd_dpp_state <= ST_RD_DPP_IDLE ;
			buf_in_ready <= 1 ;
		end		
		if ( request == GET_DEVICE_DESC ) begin
			device_desc_update <= 0 ;			
			if ( rd_dpp_addr == 0 ) begin
				{bLength,bDescriptorType,bcdUSB[7:0],bcdUSB[15:8]} <= dpp_data ;
				device_desc_update <= 1 ;
			end
		end
		else if ( request == GET_PART_CONFIG_DESC ) begin
			config_desc_update <= 0 ; 
			if ( rd_dpp_addr == 0 ) begin
				{bLength,bDescriptorType,wTotalLength[7:0],wTotalLength[15:8]} <= dpp_data ;
				config_desc_update <= 1 ;
			end
		end
		else if ( request == GET_FULL_CONFIG_DESC ) begin
			
		end
		else if ( request == GET_PART_BOS_DESC ) begin
			bos_desc_update <= 0 ;
			if ( rd_dpp_addr == 0 ) begin
				{bLength,bDescriptorType,wTotalLength[7:0],wTotalLength[15:8]} <= dpp_data ;	
				bos_desc_update <= 1 ;
			end
		end
		else if ( request == GET_FULL_BOS_DESC  ) begin
		
		end
		else if ( request == SET_DEVICE_CONFIG ) begin
		
		end
		else if ( request == GET_DEV_INTF ) begin
			alt_setting_update <= 0 ;
			if ( rd_dpp_addr == 0 ) begin
				alt_setting_update <= 1 ;
				alt_setting <= dpp_data [31:24] ;
			end
		end
	end 
	endcase 
	
	if ( config_desc_update ) begin
		config_total_len <= wTotalLength ;
	end
	if ( bos_desc_update ) begin
		bos_total_len <= wTotalLength ;
	end


	
end




reg [31:0] out_mem [0:255] ;
always @ ( posedge clk  ) begin
	if ( load_request ) begin
		out_mem[0] <= { bmRequestType , bRequest , wValue[7:0] , wValue[15:8] } ; 
		out_mem[1] <= { wIndex[7:0] , wIndex[15:8] , wLength[7:0] , wLength[15:8] } ;
	end
end

always @ ( posedge clk or negedge rst_n ) begin
	if ( !rst_n ) begin
		buf_out_q <= 0 ;
	end
	else if ( transfer_active ) begin
		buf_out_q <= 32'hff_ff_ff_ff ;
		buf_out_hasdata <= 1 ;
		buf_out_len <= ep2_transfer_cnt + 1 ;
	end
	else if ( data_out_stage ) begin
		buf_out_q <= 32'hff_ff_ff_ff ;
		buf_out_hasdata <= 1 ;
		buf_out_len <= wLength ;		
	end
	else if ( tx_pkt_act ) begin
		buf_out_q <= {6'b0,buf_out_addr[9:0],6'b0,buf_out_addr[9:0]};
		buf_out_hasdata <= 1 ;
		buf_out_len <= prot_tx_dph_len ;
	end
	else begin
		buf_out_q <= out_mem [buf_out_addr] ;
	end
end

reg [31:0] in_mem [0:255] ;
always @ ( posedge clk ) begin
	in_mem [buf_in_addr] <= buf_in_data ;
end


always @ ( posedge clk ) begin
	dpp_data <= in_mem [rd_dpp_addr] ;
end


always @ ( posedge clk ) begin
	if ( rx_dph_active && !rx_dpp_crcgood ) begin
		show_time();
		$finish();
	end
end

reg [15:0] cnt ;
reg mask = 0 ;
always @ ( posedge clk ) begin
	if ( prot_rx_dph ) begin
		show_time();
		$display ("host received data pkt \n");
		$display ("data pkt length is %d bytes : \n" , prot_rx_dph_len );
	end
	if ( buf_in_wren && !loopback ) begin
		cnt <= cnt + 4 ;
		if (  cnt +  4 < prot_rx_dph_len ) begin 
			$display ("%h \n",buf_in_data);
		end
		else if ( cnt +  4 >= prot_rx_dph_len && !mask ) begin
			mask <= 1 ;
			case ( prot_rx_dph_len [1:0] )
			0:begin $display ("%h \n",buf_in_data);	end
			1:begin $display ("%h \n",buf_in_data[31:24]);	end
			2:begin $display ("%h \n",buf_in_data[31:16]);	end
			3:begin $display ("%h \n",buf_in_data[31:8]);	end
			default:begin end
			endcase 
		end
	end 
	else begin
		cnt <= 0 ;
		mask <= 0 ;
	end
	
end


integer i ;
integer j ;

	
task loopback_test();


	begin
		$display (" loopback test begins   \n") ;
		
		out_endpt_seq_num <= 0 ;
		in_endpt_seq_num <= 0 ;
		tx_pkt_cnt <= 0 ;
		rx_pkt_cnt <= 0 ;
		tx_pkt_act <= 0 ;
		
		endpt_num <= 1 ;
		
		@ ( posedge clk ) ;
		
		for ( j = 0 ; j < 15 ; j = j + 1 ) begin 
		
			for ( i = 0 ; i < 1025 ; i = i + 1  ) begin
			
				@ ( posedge clk ) ;
				
				tx_pkt_act <= 1 ;
			
				tx_pkt ( endpt_num );
						
				tx_pkt_act <= 0 ; 		
				
				#200;
				
				rx_pkt_act <= 1 ;
			
				rx_pkt ( endpt_num);
				
				rx_pkt_act <= 0 ; 
				
				tx_pkt_cnt <= tx_pkt_cnt + 1 ;	
				
				wait ( buf_in_wren == 0 ) ;
			
			end
			
			out_endpt_seq_num <= 0 ;
			in_endpt_seq_num <= 0 ;
			tx_pkt_cnt <= 0 ;
			rx_pkt_cnt <= 0 ;
			tx_pkt_act <= 0 ;	

			endpt_num <= endpt_num + 1 ;			
		
		end
		
		$display (" loopback test finished   \n") ;
		
		$finish();
		
	end


endtask


task tx_pkt ;
input [3:0] tx_endpt_num  ;

	begin
		@ ( posedge clk ) ;
		
		prot_tx_dph			<= 1 ;
		prot_tx_dph_eob		<= 0 ; 
		prot_tx_dph_dir		<= 0 ; 
		prot_tx_dph_endp	<= tx_endpt_num[3:0] ;
		prot_tx_dph_seq		<= out_endpt_seq_num ;
		prot_tx_dph_len		<= tx_pkt_cnt; 
		prot_tx_dph_setup   <= 0  ;

		
		
		tx_pkt_act <= 1 ;
		
		wait ( prot_tx_dpp_ack ) begin
			out_endpt_seq_num <= out_endpt_seq_num + 1 ; 
			prot_tx_dph <= 0 ;
		end
		
		@ ( posedge prot_tx_dpp_done ) ;
		
		wait ( prot_rx_tp == 1 && prot_rx_tp_subtype == ACK && prot_rx_tp_endp == tx_endpt_num  ) ;
		
	end 


endtask


reg data_pkt_recieved = 0 ;

task rx_pkt  ;

input [3:0] rx_endpt_num ;



	begin
		@ ( posedge clk ) ;
		while ( !data_pkt_recieved ) begin
			prot_tx_tp_a		 <= 1;
			prot_tx_tp_a_retry	 <= 0;
			prot_tx_tp_a_dir	 <= 1;
			prot_tx_tp_a_subtype <= ACK;
			prot_tx_tp_a_endp	 <= rx_endpt_num ; 
			prot_tx_tp_a_nump    <= 1;
			prot_tx_tp_a_seq	 <= in_endpt_seq_num;
			prot_tx_tp_a_stream  <= 0;
			prot_tx_tp_a_pp		 <= 1;
			
			wait ( prot_tx_tp_a_ack ) begin
				prot_tx_tp_a <= 0 ;
			end
			
			wait ( prot_rx_dph == 1 || prot_rx_tp == 1 ) begin
				@ ( posedge clk );
				if ( prot_rx_tp == 1  ) begin
					if ( prot_rx_tp_subtype == NRDY && prot_rx_tp_endp == rx_endpt_num ) begin
						wait ( prot_rx_tp_subtype == ERDY && prot_rx_tp_endp == rx_endpt_num ) ;
					end
					else begin
						show_time();
						$display("Loopback error\n");
						$finish();
					end
				end	
				else begin
					if ( prot_rx_dph && prot_rx_dph_seq != in_endpt_seq_num ) begin
						show_time();
						$display("Loopback error\n");
						$finish();					
					end
					rx_dpp_act <= 1 ; 
					wait ( rx_dpp_act_end ) ;
					data_pkt_recieved <= 1 ;
					in_endpt_seq_num <= in_endpt_seq_num + 1 ;
					@ ( posedge clk ) ;
					rx_dpp_act <= 0 ;
				end
			end
		end
		
		data_pkt_recieved <= 0 ; 
	end

endtask

reg [15:0] rx_cnt = 0;
always @ ( posedge clk ) begin
	if ( rx_dpp_act ) begin
		if ( buf_in_wren ) begin
			rx_cnt  <= rx_cnt + 4 ;
			
			if ( prot_rx_dph_len != tx_pkt_cnt  ) begin
				show_time();
				$display("Loopback error\n");  
				$finish();
			end
		
			if ( prot_rx_dph_len != 0 ) begin 
				if ( rx_cnt + 4 < prot_rx_dph_len ) begin
					if ( buf_in_data != {6'b0,buf_in_addr[9:0],6'b0,buf_in_addr[9:0]} ) begin
						show_time();
						$display("Loopback error\n"); 
						$finish();
					end
				end
				else if ( rx_cnt + 4 >= prot_rx_dph_len && !rx_dpp_act_end ) begin
					rx_dpp_act_end <= 1 ; 
					case ( prot_rx_dph_len[1:0] )
					0:begin
						if ( buf_in_data[31:0] != {6'b0,buf_in_addr[9:0],6'b0,buf_in_addr[9:0]} ) begin
							show_time();
							$display("Loopback error\n"); 
							$finish();
						end				
					end
					1:begin
						if ( buf_in_data[31:24] != {6'b0,buf_in_addr[9:8]} ) begin
							show_time();
							$display("Loopback error\n"); 
							$finish();
						end						
					end
					2:begin
						if ( buf_in_data[31:16] != {6'b0,buf_in_addr[9:0]}) begin
							show_time();
							$display("Loopback error\n"); 
							$finish();
						end					
					end
					3:begin
						if ( buf_in_data[31:8] != {6'b0,buf_in_addr[9:0],6'b0,buf_in_addr[9:8]}) begin
							show_time();
							$display("Loopback error\n"); 
							$finish();
						end					
					end			
					default:begin end
					endcase 
				end
			end
			else begin
				rx_dpp_act_end <= 1 ;
			end
		end
		else begin
			rx_dpp_act_end <= 0 ;
		end
	end
	else begin
		rx_dpp_act_end <= 0 ;
	end
end


task show_time ;
	begin
		$display("at time %t ns\n",$realtime/1000);
	end
endtask

endmodule